// CC BY
// code by SASANO Takayoshi

#include "io.h"
#include "sprite.h"
#include "image.h"
#include "misc.h"

static	const unsigned char	ascii2sprite[] =
{
	0x00, 0xa4, 0x00, 0xa0, 0x00, 0xa2, 0x00, 0x00,
	0x00, 0x00, 0xa1, 0x9a, 0x9e, 0x9b, 0x9d, 0xa3,
	0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad,
	0xae, 0xaf, 0x9f, 0x00, 0x00, 0x9c, 0x00, 0xa5,
	0x00, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86,
	0x87, 0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e,
	0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96,
	0x97, 0x98, 0x99, 0x00, 0x00, 0x00, 0x00, 0x00,
};

static void	display_char(unsigned char c, unsigned char pos)
{
	const unsigned char	*p;
	unsigned char	x, y, c1, c2, bg;
	unsigned short	d;

	if (c < 0x20) c = 0;
	if (c >= 'a' && c <= 'z') c -= 0x20;
	c -= 0x20;
	c = (c < sizeof(ascii2sprite)) ? ascii2sprite[c] : 0;

	if (c) {
		p = get_sprite_data(c);
		bg = sprite_bgcolor(c);
	} else {
		p = NULL;
		bg = ~0;
	}

	set_es(0xa000);
	for (y = 0; y < 8; y++) {
		for (x = 0; x < 8; x += 2) {
			d = (y + 192) * 320 + x + pos * 8;
			if (p == NULL) {
				poke_w(d, 0x0000);
			} else {
				c1 = *p & 0x0f;
				c2 = (*p & 0xf0) >> 4;

				if (c1 == bg) c1 = 0;
				if (c2 == bg) c2 = 0;

				poke_w(d, (c1 << 8) | c2);
				p++;
			}
		}
	}
}

void	debug_put_string(const unsigned char *str)
{
	unsigned char	i;

	for (i = 0; i < 32; i++) {
		if (*str == '\0') break;
		display_char(*str++, i);
	}
	for (; i < 32; i++) {
		display_char('\0', i);
	}
}

void	debug_put_hex(unsigned char v)
{
	static const char	hex[] = "0123456789ABCDEF";
	unsigned char	buf[5];

	buf[0] = v;
	buf[1] = ' ';
	buf[2] = hex[(v >> 4) & 0x0f];
	buf[3] = hex[(v >> 0) & 0x0f];
	buf[4] = '\0';

	debug_put_string(buf);
}
