// CC BY
// code by SASANO Takayoshi

#include <stdio.h>
#include <stdlib.h>

struct palette {
	unsigned char	red;
	unsigned char	green;
	unsigned char	blue;
} __attribute__((packed));

static	const struct palette	Palette[] = {
	{  0,   0,   0},
	{255, 255, 255},
	{227,  27,  41},
	{255, 104, 139},
	{242, 126,  48},
	{255, 203,  61},
	{255, 222, 169},
	{107,  74,  43},
	{145, 202,  24},
	{ 28,  77,  55},
	{ 67, 175, 215},
	{ 38,  74, 208},
	{ 26,  36, 102},
	{ 99,  55, 187},
	{178,  63, 171},
	{204, 204, 204},
};

static	void	*load_file(char *filename)
{
	FILE	*fp;
	long	size;
	void	*buf = NULL;

	/* file open */
	fp = fopen(filename, "r");
	if (fp == NULL) {
		fprintf(stderr, "file open error (%s)\n", filename);
		return NULL;
	}

	/* get file size */
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	/* allocate memory */
	buf = malloc(size);
	if (buf == NULL) {
		fprintf(stderr, "memory allocation error\n");
		return NULL;
	}

	/* file load */
	if (fread(buf, 1, size, fp) < size) {
		fprintf(stderr, "file read error (%s)\n", filename);
	}

	fclose(fp);
	return buf;
}

static	int	find_index(struct palette *c)
{
	int	i, out, xr, xg, xb;
	unsigned int	d, d0;
	const struct	palette *p = Palette;

	out = -1;
	d0 = ~0;
	for (i = 0; i < (sizeof(Palette) / sizeof(Palette[0])); i++) {
		xr = p->red - c->red;
		xg = p->green - c->green;
		xb = p->blue - c->blue;

		d = (xr * xr) + (xg * xg) + (xb * xb);

		if (d < d0) {
			out = i;
			d0 = d;
		}

		p++;
	}

	return out;
}

int	main(int argc, char *argv[])
{
	int	x, y, in_x, in_y, out_x, out_y, step_x, step_y, c0, c1;
	struct	palette	*px;

	if (argc < 6) {
		fprintf(stderr, "usage: %s [in_x] [in_y] [out_x] [out_y] "
			"[filename]\n", argv[0]);
		goto fin;
	}

	px = load_file(argv[5]);
	if (px == NULL) goto fin;

	in_x = atoi(argv[1]);
	in_y = atoi(argv[2]);
	out_x = atoi(argv[3]);
	out_y = atoi(argv[4]);

	step_x = in_x / out_x;
	step_y = in_y / out_y;

	printf("\t/* %s */\n", argv[5]);
	for (y = 0; y < in_y; y += step_y) {
		printf("\t");
		for (x = 0; x < in_x; x += step_x * 2) {
			c0 = find_index(&px[y * in_x + x]);
			c1 = find_index(&px[y * in_x + x + step_x]);

			if (c0 < 0 || c1 < 0) {
				fprintf(stderr, "color convert error (%s)\n",
					argv[5]);
			}

			printf("0x%x%x, ", (c0 & 0x0f), (c1 & 0x0f));
		}
		printf("\n");
	}
fin:
	return 0;
}
