#!/bin/sh

IMAGE_URL="http://pcn.club/ns/pan_image/image/"
SPRITE_URL="http://pcn.club/ns/pan_image/sprite/"

IMAGE_LIST="list.image"
SPRITE_LIST="list.sprite"

IMAGE_SOURCE="../image.c"

IMAGE_PNG=`cat $IMAGE_LIST`
SPRITE_PNG=`cat $SPRITE_LIST`

# fetch all PNGs
for i in $IMAGE_PNG
do
    echo $IMAGE_URL$i | wget -i -
done

for i in $SPRITE_PNG
do
    echo $SPRITE_URL$i | wget -i -
done

# convert all PNG files to RGB
for i in $IMAGE_PNG $SPRITE_PNG
do
    convert ${i} ${i%.png}.rgb
done

# make RGB to C file converter
gcc -Wall dump.c -O2 -o dump

# create source header
echo "// CC BY-NC-SA" > $IMAGE_SOURCE
echo "// original data by PCN" >> $IMAGE_SOURCE
echo "// this file is automatically-genetated, do not edit." >> $IMAGE_SOURCE
echo "" >> $IMAGE_SOURCE

# convert image file
echo "const static unsigned char image_resource[] = {" >> $IMAGE_SOURCE
for i in $IMAGE_PNG
do
    ./dump 160 90 80 45 ${i%.png}.rgb >> $IMAGE_SOURCE
done
echo "};" >> $IMAGE_SOURCE

# convert sprite file
echo "const static unsigned char sprite_resource[] = {" >> $IMAGE_SOURCE
for i in $SPRITE_PNG
do
    ./dump 32 32 8 8 ${i%.png}.rgb >> $IMAGE_SOURCE
done
echo "};" >> $IMAGE_SOURCE

# trailer
echo "" >> $IMAGE_SOURCE
wc -l $IMAGE_LIST |awk '{printf ("#define\tIMAGE_ENTRY\t%d\n", $1)}' >> $IMAGE_SOURCE
echo "const unsigned char *image_data = image_resource;" >> $IMAGE_SOURCE
echo "const unsigned short image_size = sizeof(image_resource) / IMAGE_ENTRY;" >> $IMAGE_SOURCE
echo "const unsigned short image_entry = IMAGE_ENTRY;" >> $IMAGE_SOURCE
echo "" >> $IMAGE_SOURCE
wc -l $SPRITE_LIST |awk '{printf ("#define\tSPRITE_ENTRY\t%d\n", $1)}' >> $IMAGE_SOURCE
echo "const unsigned char *sprite_data = sprite_resource;" >> $IMAGE_SOURCE
echo "const unsigned short sprite_size = sizeof(sprite_resource) / SPRITE_ENTRY;" >> $IMAGE_SOURCE
echo "const unsigned short sprite_entry = SPRITE_ENTRY;" >> $IMAGE_SOURCE
echo "" >> $IMAGE_SOURCE

# clean up
for i in $IMAGE_PNG $SPRITE_PNG
do
    rm -f $i ${i%.png}.rgb
done

rm -f dump
