// CC BY
// code by SASANO Takayoshi

#include "io.h"

#define	PIC0_COMMAND	0x20
#define	PIC0_DATA	0x21
#define	PIC1_COMMAND	0xa0
#define	PIC1_DATA	0xa1

#define	IRQ2PICREG(irq)	((irq < 8) ? PIC0_DATA : PIC1_DATA)

void	enable_irq(unsigned char irq)
{
	unsigned short	reg;

	reg = IRQ2PICREG(irq);

	out_b(reg, in_b(reg) & ~(1 << (irq & 7)));
}

void	disable_irq(unsigned char irq)
{
	unsigned short	reg;

	reg = IRQ2PICREG(irq);

	out_b(reg, in_b(reg) | (1 << (irq & 7)));
}

void	send_eoi(unsigned char irq)
{
	if (irq >= 8) {
		out_b(PIC1_COMMAND, 0x20);
	}

	out_b(PIC0_COMMAND, 0x20);
}

static	unsigned short	ivt_offset(unsigned char irq)
{
	unsigned short	offset;

	offset = irq;
	offset += (irq < 8) ? 0x08 : 0x68;
	offset *= 4;

	return offset;
}

unsigned long	get_irq_vector(unsigned char irq)
{
	unsigned short	offset;

	offset = ivt_offset(irq);

	set_es(0);
	return (peek_w(offset + 2) << 16) | peek_w(offset);
}

void	set_irq_vector(unsigned char irq, unsigned long vector)
{
	unsigned short	offset;

	offset = ivt_offset(irq);

	set_es(0);
	poke_w(offset, vector);
	poke_w(offset + 2, vector >> 16);
}
