// CC BY
// code by SASANO Takayoshi

#include "gfx.h"
#include "image.h"
#include "misc.h"

#define	MAX_SPRITE	16
#define	VALID_BG_MAX	0x1f
#define	isSpriteMode(x)	((x) <= VALID_BG_MAX)

struct sprite {
	const unsigned char	*pattern;
	unsigned char	bgcolor;
	signed char	x;
	signed char	y;
};

static	struct sprite	Sprite[MAX_SPRITE];
static	unsigned char	SpriteBG;

static	unsigned char	sprite_bgcolor_workaround(unsigned char pattern)
{
	/* XXX need more clever workaround */
	switch (pattern) {
	case 0x01:
	case 0x08:
	case 0x09:
		pattern = 0x00;
		break;
	case 0x6d:
	case 0x6e:
	case 0x6f:
		pattern = 0x68;
		break;
	case 0x44:
	case 0x5d:
	case 0x5e:
	case 0x5f:
	case 0x65:
	case 0x66:
	case 0x67:
		pattern = ~0;
		break;
	}

	return pattern;
}

unsigned char	sprite_bgcolor(unsigned char pattern)
{
	unsigned char	c0, c1, c2;
	const unsigned char	*p;

	pattern = sprite_bgcolor_workaround(pattern);

	p = get_sprite_data(pattern);
	if (p == NULL) {
		return ~0;
	}

	c0 = (p[0] >> 4) & 0x0f;	// left top
	c1 = p[3] & 0x0f;		// right top
	c2 = p[28] & 0x0f;		// left bottom

	if (c0 == c1) return c0;
	else if (c1 == c2) return c1;
	else if (c2 == c0) return c2;
	else return ~0;
}

void	sprite_define(unsigned char id, unsigned char pattern)
{
	if (id >= MAX_SPRITE) {
		return;
	}

	Sprite[id].pattern = get_sprite_data(pattern);
	Sprite[id].bgcolor = sprite_bgcolor(pattern);
}

void	sprite_move(unsigned char id, signed char x, signed char y)
{
	if (id >= MAX_SPRITE) {
		return;
	}

	Sprite[id].x = x;
	Sprite[id].y = y;
}

static	void	sprite_bg_setup(void)
{
	if (isValidImageEntry(SpriteBG)) {
		gfx_display_image(get_image_data(SpriteBG & 0x0f));
	} else {
		gfx_clear(SpriteBG & 0x0f);
	}
}

void	sprite_render(void)
{
	unsigned short	i;

	/* sprite mode is disabled: quit immediately */
	if (!isSpriteMode(SpriteBG)) {
		return;
	}

	/* BG screen */
	sprite_bg_setup();

	/* put sprite(s) */
	for (i = 0; i < MAX_SPRITE; i++) {
		gfx_draw_block8x8(Sprite[i].x, Sprite[i].y,
				  Sprite[i].bgcolor, Sprite[i].pattern);
	}
}

void	sprite_init(void)
{
	unsigned short	i;

	SpriteBG = ~0;

	for (i = 0; i < MAX_SPRITE; i++) {
		Sprite[i].pattern = NULL;
		Sprite[i].bgcolor =~0;
		Sprite[i].x = Sprite[i].y = -127;
	}
}

void	sprite_set_bg(unsigned char id)
{
	/* quit sprite mode */
	if (isSpriteMode(SpriteBG) && !isSpriteMode(id)) {
		sprite_bg_setup();
	}

	SpriteBG = id;
}

unsigned char	sprite_is_enable(void)
{
	return isSpriteMode(SpriteBG);
}
