// CC BY
// code by SASANO Takayoshi

#include "io.h"
#include "gfx.h"

struct palette {
	unsigned char	red;
	unsigned char	green;
	unsigned char	blue;
} __attribute__((packed));

static	const struct palette	Palette[] = {
	{  0,   0,   0},
	{255, 255, 255},
	{227,  27,  41},
	{255, 104, 139},
	{242, 126,  48},
	{255, 203,  61},
	{255, 222, 169},
	{107,  74,  43},
	{145, 202,  24},
	{ 28,  77,  55},
	{ 67, 175, 215},
	{ 38,  74, 208},
	{ 26,  36, 102},
	{ 99,  55, 187},
	{178,  63, 171},
	{204, 204, 204},
};

static	unsigned char	DisplayEnable;

static	void	vram_clear(void)
{
	unsigned short	i;

	set_es(0xa000);
	i = 0;
	do {
		poke_w(i, 0);
		i += 2;
	} while (i);
}

static	void	vram_update(void)
{
	unsigned short	i, x, y, d, c;

	set_es(0xa000);
	for (y = 0; y < GFX_Y_SIZE; y++) {
		for (x = 0; x < GFX_X_SIZE; x++) {
			/* magnify 4x4 */
			d = x * 4 + ((y * 4) + 10) * 320;
			c = gfx_get_pixel(x, y);
			c |= (c << 8);

			for (i = 0; i < 4; i++) {
				/* some VGA does not want 32bit access */
				poke_w(d + 0, c);
				poke_w(d + 2, c);
				d += 320;
			}
		}
	}
}

void	video_init(void)
{
	unsigned short	i;

	/* set video mode to 13H */
	__asm volatile("int $0x10" :: "a"(0x13));

	/* clear Video-RAM */
	vram_clear();

	/* set palette */
	DI();
	for (i = 0; i < 256; i++) {
		/* DAC is 6bit */
		out_b(0x3c8, i);
		out_b(0x3c9, Palette[i & 0x0f].red >> 2);
		out_b(0x3c9, Palette[i & 0x0f].green >> 2);
		out_b(0x3c9, Palette[i & 0x0f].blue >> 2);
	}
	EI();

	/* enable display */
	DisplayEnable = 1;
}

void	video_update(void)
{
	/* display is disabled */
	if (!DisplayEnable) {
		return;
	}

	/* update */
	vram_update();
}

void	video_enable(unsigned char endis)
{
	if (endis) {
		vram_update();
	} else {
		vram_clear();
	}

	DisplayEnable = endis;
}
